@@ -1,7 +1,8 @@
 package App::Prove::State::Result::Test;
 
 use strict;
-use warnings;
+
+use vars qw($VERSION);
 
 =head1 NAME
 
@@ -9,11 +10,11 @@ App::Prove::State::Result::Test - Individual test results.
 
 =head1 VERSION
 
-Version 3.51_01
+Version 3.14
 
 =cut
 
-our $VERSION = '3.51_01';
+$VERSION = '3.14';
 
 =head1 DESCRIPTION
 
@@ -24,7 +25,7 @@ single test.
 =head1 SYNOPSIS
 
     # Re-run failed tests
-    $ prove --state=failed,save -rbv
+    $ prove --state=fail,save -rbv
 
 =cut
 
@@ -41,7 +42,6 @@ my %methods = (
     seq            => { method => 'sequence', default => 1 },
     total_passes   => { method => 'total_passes', default => 0 },
     total_failures => { method => 'total_failures', default => 0 },
-    parser         => { method => 'parser' },
 );
 
 while ( my ( $key, $description ) = each %methods ) {
@@ -132,20 +132,14 @@ The number of times the test has passed.
 
 The number of times the test has failed.
 
-=head3 C<parser>
-
-The underlying parser object.  This is useful if you need the full
-information for the test program.
-
 =cut
 
 sub raw {
     my $self = shift;
     my %raw  = %$self;
 
-    # this is backwards-compatibility hack and is not guaranteed.
+    # this is backwards-compatibility hack and is not gauranteed.
     delete $raw{name};
-    delete $raw{parser};
     return \%raw;
 }
 
